@@ -346,6 +346,7 @@ def gridsearch_mo_optim(x, y, t, n_splits=5,  param_grid=None):
 
     results = []
     for params in grid:
+
         copy_several_times = params['copy_several_times']
         temp_results = []
 
@@ -394,7 +395,7 @@ def gridsearch_mo_optim(x, y, t, n_splits=5,  param_grid=None):
 
 
     x, utility_weights, new_response, big_y  = prepare_data_optimized_loss(x,y,t,unique_treatments,
-    weighted_treatments = True, copy_several_times = copy_several_times)
+    weighted_treatments = True, copy_several_times = optim_grid['copy_several_times'])
 
     mod.fit([x, utility_weights] , [new_response, big_y], epochs = optim_grid['epochs'],
         batch_size = optim_grid['batch_size'], verbose = False)
